#!/bin/bash

# Requires the following: dd, tr, sed, mktemp, openssl, perl (sadly gzip CLI can not operate on raw zlib data and most OpenSSL distros do not include zlib)
# However even git for windows (MSysGit) includes enough perl to make it fly :)

IV='B380E17A0C476E8A480B55253ABAE666'
KEY='DD1FBB7596D0D58BBF336ABA391366ED3ED7460849BB9EFAB9649C999863BF25'

ENCRYPT=1

if [ "$1" == "-d" ]; then
	ENCRYPT=0
else
	if [ "$1" != "-e" ]; then
		echo "Usage: ./spcrypt.sh [-e / -d] [input] [output]"
		exit 1
	fi
fi
IN_FILE="$2"
OUT_FILE="$3"

if [ "$ENCRYPT" == "1" ]; then
	# Make temp file and put GZip data in
	TEMP_GZIP="$(mktemp)"
        TEMP_OUT="$(mktemp)"
	sed 's/\r\n/\n/g' "$IN_FILE" | sed 's/\r/\n/g' | perl -MCompress::Zlib -e 'undef $/; print compress(<>, 9)' > "$TEMP_GZIP"

	# Calculate padding
	INPUT_LEN="$(stat -f %z "$TEMP_GZIP")"
	INPUT_PADLEN=$(( 16 - ($INPUT_LEN % 16) ))

	# Zero-pad file
	dd if=/dev/zero of="$TEMP_GZIP" bs=1 count="$INPUT_PADLEN" seek="$INPUT_LEN" > $TEMP_OUT 2>&1

	# Encrypt
	openssl enc -aes-256-cbc -nosalt -e -K "$KEY" -iv "$IV" -nopad -out "$OUT_FILE" -in "$TEMP_GZIP"

	# Clean up temporary file
	rm -f "$TEMP_GZIP"
        rm -f "$TEMP_OUT"
else
	# Decrypt, decompress, trim zero-bytes
	openssl enc -aes-256-cbc -nosalt -d -in "$IN_FILE" -K "$KEY" -iv "$IV" -nopad | perl -MCompress::Zlib -e 'undef $/; print uncompress(<>)' | tr -d '\000' > "$OUT_FILE"
fi
